@@ -62,6 +62,7 @@ enum {
 	TRANSFER_STATUS_ACTIVE,
 	TRANSFER_STATUS_SUSPENDED,
 	TRANSFER_STATUS_COMPLETE,
+	TRANSFER_STATUS_SUSPENDED_QUEUED,
 	TRANSFER_STATUS_ERROR
 };
 
@@ -159,6 +160,9 @@ static DBusMessage *obc_transfer_cancel(DBusConnection *connection,
 				ERROR_INTERFACE ".InProgress",
 				"Cancellation already in progress");
 
+	if (transfer->status == TRANSFER_STATUS_SUSPENDED)
+		g_obex_resume(transfer->obex);
+
 	if (transfer->req > 0) {
 		if (!g_obex_cancel_req(transfer->obex, transfer->req, TRUE))
 			return g_dbus_create_error(message,
@@ -213,6 +217,7 @@ static DBusMessage *obc_transfer_suspend(DBusConnection *connection,
 {
 	struct obc_transfer *transfer = user_data;
 	const char *sender;
+	uint8_t status;
 
 	sender = dbus_message_get_sender(message);
 	if (g_strcmp0(transfer->owner, sender) != 0)
@@ -220,14 +225,22 @@ static DBusMessage *obc_transfer_suspend(DBusConnection *connection,
 				ERROR_INTERFACE ".NotAuthorized",
 				"Not Authorized");
 
-	if (transfer->xfer == 0)
+	switch (transfer->status) {
+	case TRANSFER_STATUS_QUEUED:
+		status = TRANSFER_STATUS_SUSPENDED_QUEUED;
+		break;
+	case TRANSFER_STATUS_ACTIVE:
+		if (transfer->xfer)
+			g_obex_suspend(transfer->obex);
+		status = TRANSFER_STATUS_SUSPENDED;
+		break;
+	default:
 		return g_dbus_create_error(message,
 				ERROR_INTERFACE ".NotInProgress",
 				"Not in progress");
+	}
 
-	g_obex_suspend(transfer->obex);
-
-	transfer_set_status(transfer, TRANSFER_STATUS_SUSPENDED);
+	transfer_set_status(transfer, status);
 
 	return g_dbus_create_reply(message, DBUS_TYPE_INVALID);
 }
@@ -237,6 +250,7 @@ static DBusMessage *obc_transfer_resume(DBusConnection *connection,
 {
 	struct obc_transfer *transfer = user_data;
 	const char *sender;
+	uint8_t status;
 
 	sender = dbus_message_get_sender(message);
 	if (g_strcmp0(transfer->owner, sender) != 0)
@@ -244,14 +258,24 @@ static DBusMessage *obc_transfer_resume(DBusConnection *connection,
 				ERROR_INTERFACE ".NotAuthorized",
 				"Not Authorized");
 
-	if (transfer->xfer == 0)
+	switch (transfer->status) {
+	case TRANSFER_STATUS_SUSPENDED_QUEUED:
+		status = TRANSFER_STATUS_QUEUED;
+		break;
+	case TRANSFER_STATUS_SUSPENDED:
+		if (transfer->xfer)
+			g_obex_resume(transfer->obex);
+		else
+			obc_transfer_start(transfer, NULL, NULL);
+		status = TRANSFER_STATUS_ACTIVE;
+		break;
+	default:
 		return g_dbus_create_error(message,
 				ERROR_INTERFACE ".NotInProgress",
 				"Not in progress");
+	}
 
-	g_obex_resume(transfer->obex);
-
-	transfer_set_status(transfer, TRANSFER_STATUS_ACTIVE);
+	transfer_set_status(transfer, status);
 
 	return g_dbus_create_reply(message, DBUS_TYPE_INVALID);
 }
@@ -338,6 +362,7 @@ static const char *status2str(uint8_t status)
 		return "queued";
 	case TRANSFER_STATUS_ACTIVE:
 		return "active";
+	case TRANSFER_STATUS_SUSPENDED_QUEUED:
 	case TRANSFER_STATUS_SUSPENDED:
 		return "suspended";
 	case TRANSFER_STATUS_COMPLETE:
@@ -392,6 +417,9 @@ static void obc_transfer_free(struct obc_transfer *transfer)
 {
 	DBG("%p", transfer);
 
+	if (transfer->status == TRANSFER_STATUS_SUSPENDED)
+		g_obex_resume(transfer->obex);
+
 	if (transfer->req > 0)
 		g_obex_cancel_req(transfer->obex, transfer->req, TRUE);
 
@@ -637,6 +665,9 @@ static void xfer_complete(GObex *obex, GError *err, gpointer user_data)
 		transfer->progress_id = 0;
 	}
 
+	if (transfer->status == TRANSFER_STATUS_SUSPENDED)
+		g_obex_resume(transfer->obex);
+
 	if (err)
 		transfer_set_status(transfer, TRANSFER_STATUS_ERROR);
 	else
@@ -861,7 +892,14 @@ static gboolean transfer_start_put(struct obc_transfer *transfer, GError **err)
 gboolean obc_transfer_start(struct obc_transfer *transfer, void *obex,
 								GError **err)
 {
-	transfer->obex = g_obex_ref(obex);
+	if (!transfer->obex)
+		transfer->obex = g_obex_ref(obex);
+
+	if (transfer->status == TRANSFER_STATUS_SUSPENDED_QUEUED) {
+		/* Reset status so the transfer can be resumed */
+		transfer->status = TRANSFER_STATUS_SUSPENDED;
+		return TRUE;
+	}
 
 	switch (transfer->op) {
 	case G_OBEX_OP_GET:
