@@ -36,8 +36,6 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <sys/uio.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
@@ -57,7 +55,7 @@
 
 #define TI_MANUFACTURER_ID	13
 
-#define FIRMWARE_DIRECTORY	"/lib/firmware/"
+#define FIRMWARE_DIRECTORY	"/lib/firmware/ti-connectivity/"
 
 #define ACTION_SEND_COMMAND	1
 #define ACTION_WAIT_EVENT	2
@@ -111,10 +109,10 @@ struct bts_action_serial {
 	uint32_t flow_control;
 }__attribute__ ((packed));
 
-static FILE *bts_load_script(const char* file_name, uint32_t* version)
+static FILE *bts_load_script(const char *file_name, uint32_t *version)
 {
 	struct bts_header header;
-	FILE* fp;
+	FILE *fp;
 
 	fp = fopen(file_name, "rb");
 	if (!fp) {
@@ -143,8 +141,8 @@ errclose:
 	return NULL;
 }
 
-static unsigned long bts_fetch_action(FILE* fp, unsigned char* action_buf,
-				unsigned long buf_size, uint16_t* action_type)
+static unsigned long bts_fetch_action(FILE *fp, unsigned char *action_buf,
+				unsigned long buf_size, uint16_t *action_type)
 {
 	struct bts_action action_hdr;
 	unsigned long nread;
@@ -171,7 +169,7 @@ static unsigned long bts_fetch_action(FILE* fp, unsigned char* action_buf,
 	return nread * sizeof(uint8_t);
 }
 
-static void bts_unload_script(FILE* fp)
+static void bts_unload_script(FILE *fp)
 {
 	if (fp)
 		fclose(fp);
@@ -239,7 +237,7 @@ static int brf_set_serial_params(struct bts_action_serial *serial_action,
 	return 0;
 }
 
-static int brf_send_command_socket(int fd, struct bts_action_send* send_action)
+static int brf_send_command_socket(int fd, struct bts_action_send *send_action)
 {
 	char response[1024] = {0};
 	hci_command_hdr *cmd = (hci_command_hdr *) send_action->data;
@@ -269,7 +267,8 @@ static int brf_send_command_socket(int fd, struct bts_action_send* send_action)
 	return 0;
 }
 
-static int brf_send_command_file(int fd, struct bts_action_send* send_action, long size)
+static int brf_send_command_file(int fd, struct bts_action_send *send_action,
+								long size)
 {
 	unsigned char response[1024] = {0};
 	long ret = 0;
@@ -298,7 +297,8 @@ static int brf_send_command_file(int fd, struct bts_action_send* send_action, lo
 }
 
 
-static int brf_send_command(int fd, struct bts_action_send* send_action, long size, int hcill_installed)
+static int brf_send_command(int fd, struct bts_action_send *send_action,
+						long size, int hcill_installed)
 {
 	int ret = 0;
 	char *fixed_action;
@@ -322,7 +322,9 @@ static int brf_do_action(uint16_t brf_type, uint8_t *brf_action, long brf_size,
 	switch (brf_type) {
 	case ACTION_SEND_COMMAND:
 		DPRINTF("W");
-		ret = brf_send_command(fd, (struct bts_action_send*) brf_action, brf_size, hcill_installed);
+		ret = brf_send_command(fd,
+					(struct bts_action_send *) brf_action,
+					brf_size, hcill_installed);
 		break;
 	case ACTION_WAIT_EVENT:
 		DPRINTF("R");
